@@ -37,51 +37,56 @@ DocReader_init(DocReader *self, Schema *schema, Folder *folder,
 }
 
 DocReader*
-DocReader_aggregator(DocReader *self, VArray *readers, I32Array *offsets) {
+DocReader_Aggregator_IMP(DocReader *self, VArray *readers,
+                         I32Array *offsets) {
     UNUSED_VAR(self);
     return (DocReader*)PolyDocReader_new(readers, offsets);
 }
 
 PolyDocReader*
 PolyDocReader_new(VArray *readers, I32Array *offsets) {
-    PolyDocReader *self = (PolyDocReader*)VTable_Make_Obj(POLYDOCREADER);
+    PolyDocReader *self = (PolyDocReader*)Class_Make_Obj(POLYDOCREADER);
     return PolyDocReader_init(self, readers, offsets);
 }
 
 PolyDocReader*
 PolyDocReader_init(PolyDocReader *self, VArray *readers, I32Array *offsets) {
     DocReader_init((DocReader*)self, NULL, NULL, NULL, NULL, -1);
+    PolyDocReaderIVARS *const ivars = PolyDocReader_IVARS(self);
     for (uint32_t i = 0, max = VA_Get_Size(readers); i < max; i++) {
         CERTIFY(VA_Fetch(readers, i), DOCREADER);
     }
-    self->readers = (VArray*)INCREF(readers);
-    self->offsets = (I32Array*)INCREF(offsets);
+    ivars->readers = (VArray*)INCREF(readers);
+    ivars->offsets = (I32Array*)INCREF(offsets);
     return self;
 }
 
 void
-PolyDocReader_close(PolyDocReader *self) {
-    if (self->readers) {
-        for (uint32_t i = 0, max = VA_Get_Size(self->readers); i < max; i++) {
-            DocReader *reader = (DocReader*)VA_Fetch(self->readers, i);
+PolyDocReader_Close_IMP(PolyDocReader *self) {
+    PolyDocReaderIVARS *const ivars = PolyDocReader_IVARS(self);
+    if (ivars->readers) {
+        for (uint32_t i = 0, max = VA_Get_Size(ivars->readers); i < max; i++) {
+            DocReader *reader = (DocReader*)VA_Fetch(ivars->readers, i);
             if (reader) { DocReader_Close(reader); }
         }
-        VA_Clear(self->readers);
+        VA_Clear(ivars->readers);
     }
 }
 
 void
-PolyDocReader_destroy(PolyDocReader *self) {
-    DECREF(self->readers);
-    DECREF(self->offsets);
+PolyDocReader_Destroy_IMP(PolyDocReader *self) {
+    PolyDocReaderIVARS *const ivars = PolyDocReader_IVARS(self);
+    DECREF(ivars->readers);
+    DECREF(ivars->offsets);
     SUPER_DESTROY(self, POLYDOCREADER);
 }
 
 HitDoc*
-PolyDocReader_fetch_doc(PolyDocReader *self, int32_t doc_id) {
-    uint32_t seg_tick = PolyReader_sub_tick(self->offsets, doc_id);
-    int32_t  offset   = I32Arr_Get(self->offsets, seg_tick);
-    DocReader *doc_reader = (DocReader*)VA_Fetch(self->readers, seg_tick);
+PolyDocReader_Fetch_Doc_IMP(PolyDocReader *self, int32_t doc_id) {
+    PolyDocReaderIVARS *const ivars = PolyDocReader_IVARS(self);
+    uint32_t seg_tick = PolyReader_sub_tick(ivars->offsets, doc_id);
+    int32_t  offset   = I32Arr_Get(ivars->offsets, seg_tick);
+    DocReader *doc_reader = (DocReader*)VA_Fetch(ivars->readers, seg_tick);
     HitDoc *hit_doc = NULL;
     if (!doc_reader) {
         THROW(ERR, "Invalid doc_id: %i32", doc_id);
@@ -97,29 +102,31 @@ DefaultDocReader*
 DefDocReader_new(Schema *schema, Folder *folder, Snapshot *snapshot,
                  VArray *segments, int32_t seg_tick) {
     DefaultDocReader *self
-        = (DefaultDocReader*)VTable_Make_Obj(DEFAULTDOCREADER);
+        = (DefaultDocReader*)Class_Make_Obj(DEFAULTDOCREADER);
     return DefDocReader_init(self, schema, folder, snapshot, segments,
                              seg_tick);
 }
 
 void
-DefDocReader_close(DefaultDocReader *self) {
-    if (self->dat_in != NULL) {
-        InStream_Close(self->dat_in);
-        DECREF(self->dat_in);
-        self->dat_in = NULL;
+DefDocReader_Close_IMP(DefaultDocReader *self) {
+    DefaultDocReaderIVARS *const ivars = DefDocReader_IVARS(self);
+    if (ivars->dat_in != NULL) {
+        InStream_Close(ivars->dat_in);
+        DECREF(ivars->dat_in);
+        ivars->dat_in = NULL;
     }
-    if (self->ix_in != NULL) {
-        InStream_Close(self->ix_in);
-        DECREF(self->ix_in);
-        self->ix_in = NULL;
+    if (ivars->ix_in != NULL) {
+        InStream_Close(ivars->ix_in);
+        DECREF(ivars->ix_in);
+        ivars->ix_in = NULL;
     }
 }
 
 void
-DefDocReader_destroy(DefaultDocReader *self) {
-    DECREF(self->ix_in);
-    DECREF(self->dat_in);
+DefDocReader_Destroy_IMP(DefaultDocReader *self) {
+    DefaultDocReaderIVARS *const ivars = DefDocReader_IVARS(self);
+    DECREF(ivars->ix_in);
+    DECREF(ivars->dat_in);
     SUPER_DESTROY(self, DEFAULTDOCREADER);
 }
 
@@ -130,14 +137,15 @@ DefDocReader_init(DefaultDocReader *self, Schema *schema, Folder *folder,
     Segment *segment;
     DocReader_init((DocReader*)self, schema, folder, snapshot, segments,
                    seg_tick);
+    DefaultDocReaderIVARS *const ivars = DefDocReader_IVARS(self);
     segment = DefDocReader_Get_Segment(self);
-    metadata = (Hash*)Seg_Fetch_Metadata_Str(segment, "documents", 9);
+    metadata = (Hash*)Seg_Fetch_Metadata_Utf8(segment, "documents", 9);
 
     if (metadata) {
-        CharBuf *seg_name  = Seg_Get_Name(segment);
-        CharBuf *ix_file   = CB_newf("%o/documents.ix", seg_name);
-        CharBuf *dat_file  = CB_newf("%o/documents.dat", seg_name);
-        Obj     *format    = Hash_Fetch_Str(metadata, "format", 6);
+        String *seg_name  = Seg_Get_Name(segment);
+        String *ix_file   = Str_newf("%o/documents.ix", seg_name);
+        String *dat_file  = Str_newf("%o/documents.dat", seg_name);
+        Obj     *format   = Hash_Fetch_Utf8(metadata, "format", 6);
 
         // Check format.
         if (!format) { THROW(ERR, "Missing 'format' var"); }
@@ -154,16 +162,16 @@ DefDocReader_init(DefaultDocReader *self, Schema *schema, Folder *folder,
 
         // Get streams.
         if (Folder_Exists(folder, ix_file)) {
-            self->ix_in = Folder_Open_In(folder, ix_file);
-            if (!self->ix_in) {
+            ivars->ix_in = Folder_Open_In(folder, ix_file);
+            if (!ivars->ix_in) {
                 Err *error = (Err*)INCREF(Err_get_error());
                 DECREF(ix_file);
                 DECREF(dat_file);
                 DECREF(self);
                 RETHROW(error);
             }
-            self->dat_in = Folder_Open_In(folder, dat_file);
-            if (!self->dat_in) {
+            ivars->dat_in = Folder_Open_In(folder, dat_file);
+            if (!ivars->dat_in) {
                 Err *error = (Err*)INCREF(Err_get_error());
                 DECREF(ix_file);
                 DECREF(dat_file);
@@ -179,18 +187,20 @@ DefDocReader_init(DefaultDocReader *self, Schema *schema, Folder *folder,
 }
 
 void
-DefDocReader_read_record(DefaultDocReader *self, ByteBuf *buffer,
-                         int32_t doc_id) {
+DefDocReader_Read_Record_IMP(DefaultDocReader *self, ByteBuf *buffer,
+                             int32_t doc_id) {
+    DefaultDocReaderIVARS *const ivars = DefDocReader_IVARS(self);
+
     // Find start and length of variable length record.
-    InStream_Seek(self->ix_in, (int64_t)doc_id * 8);
-    int64_t start = InStream_Read_I64(self->ix_in);
-    int64_t end   = InStream_Read_I64(self->ix_in);
+    InStream_Seek(ivars->ix_in, (int64_t)doc_id * 8);
+    int64_t start = InStream_Read_I64(ivars->ix_in);
+    int64_t end   = InStream_Read_I64(ivars->ix_in);
     size_t size  = (size_t)(end - start);
 
     // Read in the record.
     char *buf = BB_Grow(buffer, size);
-    InStream_Seek(self->dat_in, start);
-    InStream_Read_Bytes(self->dat_in, buf, size);
+    InStream_Seek(ivars->dat_in, start);
+    InStream_Read_Bytes(ivars->dat_in, buf, size);
     BB_Set_Size(buffer, size);
 }
 
