@@ -19,29 +19,32 @@
 #include "Lucy/Store/DirHandle.h"
 
 DirHandle*
-DH_init(DirHandle *self, const CharBuf *dir) {
-    self->dir   = CB_Clone(dir);
-    self->entry = CB_new(32);
+DH_init(DirHandle *self, String *dir) {
+    DirHandleIVARS *const ivars = DH_IVARS(self);
+    ivars->dir   = Str_Clone(dir);
+    ivars->entry = NULL;
     ABSTRACT_CLASS_CHECK(self, DIRHANDLE);
     return self;
 }
 
 void
-DH_destroy(DirHandle *self) {
+DH_Destroy_IMP(DirHandle *self) {
+    DirHandleIVARS *const ivars = DH_IVARS(self);
     DH_Close(self);
-    DECREF(self->dir);
-    DECREF(self->entry);
+    DECREF(ivars->dir);
+    DECREF(ivars->entry);
     SUPER_DESTROY(self, DIRHANDLE);
 }
 
-CharBuf*
-DH_get_dir(DirHandle *self) {
-    return self->dir;
+String*
+DH_Get_Dir_IMP(DirHandle *self) {
+    return DH_IVARS(self)->dir;
 }
 
-CharBuf*
-DH_get_entry(DirHandle *self) {
-    return self->entry;
+String*
+DH_Get_Entry_IMP(DirHandle *self) {
+    String *entry = DH_IVARS(self)->entry;
+    return (String*)INCREF(entry);
 }
 
 
